#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 8/12/22

# Inspire by Conbagui's tutorial found here: https://www.deviantart.com/conbagui/art/Donut-Text-Straberry-Effect-474255570

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import os, sys,time
import gettext
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_donut_text(customtext, fontname, backColor, topping, flacolor, linecolor, sprinkle_amnt):

    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()
    
    size = 250
    img = pdb.gimp_image_new(1, 1, RGB)
    pdb.gimp_context_set_foreground("#ca8f4d")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 100, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 10)
    #Resize text
    img.resize(textLayer.width, textLayer.height, 0, 0)
    #Rename the text layer and move up
    pdb.gimp_item_set_name(textLayer, "Text")
    textLayer = pdb.gimp_image_get_layer_by_name(img, "Text")
    x = textLayer.offsets[0]
    y = textLayer.offsets[1]
    x = (img.width - textLayer.width) / 2
    y = (img.height - textLayer.height) / 2
    textLayer.set_offsets(x, y)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    pdb.gimp_display_new(img)
    
    #Create background
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, backLayer, None, 1)
    pdb.gimp_context_set_foreground(backColor)
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
        
    #Lines pattern
    size = 20
    clothPattern = pdb.gimp_layer_new(img, size, size, RGBA_IMAGE, "Lines", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, clothPattern, None, 0)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_drawable_fill(clothPattern, FILL_TRANSPARENT)
    pdb.gimp_brushes_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(5)
    pdb.gimp_context_set_brush_aspect_ratio(0)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_brush_spacing(0.01)
    pdb.gimp_context_set_brush_force(1)
    pdb.gimp_paintbrush_default(clothPattern, 4, [0,20, 20,0])
    pdb.plug_in_make_seamless(img, clothPattern)
    non_empty = pdb.gimp_edit_copy(clothPattern)
    pdb.gimp_image_remove_layer(img, clothPattern)
    
    #Plaid pattern
    plaidPattern = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Plaid Pattern", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, plaidPattern, None, 0)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_drawable_fill(plaidPattern, FILL_TRANSPARENT)
    
    pdb.gimp_image_select_rectangle(img, 2, 160, 125, 100 , 200)
    pdb.gimp_edit_bucket_fill(plaidPattern, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_image_select_rectangle(img, 2, 110, 175, 200 , 100)
    pdb.gimp_edit_bucket_fill(plaidPattern, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    pdb.gimp_image_select_item(img, 2, plaidPattern)
    pdb.gimp_drawable_edit_clear(plaidPattern)
    pdb.gimp_context_set_pattern(_("Clipboard Image"))
    pdb.gimp_edit_bucket_fill(plaidPattern, BUCKET_FILL_PATTERN, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_image_select_rectangle(img, 2, 160, 175, 100 , 100)
    pdb.gimp_edit_bucket_fill(plaidPattern, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.gimp_selection_none(img)
    
    #Crop and save to clipboard
    pdb.gimp_image_select_item(img, 2, plaidPattern)
    x0,y0 = pdb.gimp_drawable_offsets(plaidPattern)
    non_empty, x1, y1, x2, y2 = pdb.gimp_selection_bounds(img)
    pdb.gimp_layer_resize(plaidPattern, x2-x1, y2-y1, x0-x1, y0-y1)
    pdb.gimp_selection_none(img) 
    pdb.gimp_layer_scale(plaidPattern, 100, 100, FALSE)
    non_empty = pdb.gimp_edit_copy(plaidPattern)
    pdb.gimp_image_remove_layer(img, plaidPattern)
    
    #Fill background
    #Pattern overlay		
    pattern =  _("Clipboard Image")
    opacity = 100.0
    mode = 0                       #LAYER_MODE_NORMAL_LEGACY
    scale = 100.0
    interpolation_type = INTERPOLATION_NONE
    merge = TRUE
    pdb.python_layerfx_pattern_overlay(img, backLayer, pattern, opacity, mode, scale, interpolation_type, merge)
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_item_set_name(backLayer, "Buffalo Plaid")
    
     #Drop Shadow
    pdb.script_fu_drop_shadow(img, textLayer, 8, 8, 20, "Black", 100, FALSE)
    
    #Inner shdow
    innerShad = pdb.gimp_layer_new_from_drawable(backLayer, img)
    pdb.gimp_image_insert_layer(img, innerShad, None, 0)
    pdb.gimp_item_set_name(innerShad, "Layer")
    pdb.gimp_layer_set_mode(innerShad, LAYER_MODE_NORMAL)
    pdb.gimp_layer_set_opacity(innerShad, 100)
    pdb.gimp_context_set_foreground("#775128")
    pdb.gimp_drawable_fill(innerShad, FILL_FOREGROUND)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_drawable_edit_clear(innerShad)
    pdb.gimp_selection_none(img)
    pdb.script_fu_drop_shadow(img, innerShad, 0, 8, 10, ("#775128"), 25, FALSE)
    pdb.gimp_image_remove_layer(img, innerShad)
    innerShad = pdb.gimp_image_get_layer_by_name(img, "Drop Shadow #1")
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_selection_invert(img)
    pdb.gimp_drawable_edit_clear(innerShad)
    pdb.gimp_item_set_name(innerShad, "Text inner-shadow")
    pdb.gimp_selection_none(img)
    
    #Inner Glow
    innerGlow = pdb.gimp_layer_new_from_drawable(backLayer, img)
    pdb.gimp_image_insert_layer(img, innerGlow, None, 1)
    pdb.gimp_item_set_name(innerGlow, "Layer")
    pdb.gimp_layer_set_mode(innerGlow, LAYER_MODE_SOFTLIGHT)
    pdb.gimp_layer_set_opacity(innerGlow, 75)
    pdb.gimp_item_set_name(innerShad, "Text inner-glow")
    pdb.gimp_drawable_edit_clear(innerGlow)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_context_set_foreground("#000000")
    pdb.gimp_drawable_edit_fill(innerGlow, FILL_FOREGROUND)
    pdb.gimp_selection_shrink(img, 5)
    pdb.gimp_drawable_edit_clear(innerGlow)
    pdb.gimp_selection_none(img)
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.plug_in_gauss(img, innerGlow, 14, 14, 1)   #method=1
    pdb.gimp_selection_none(img)
    
    #Bevel and emboss
    style = 1                      #Inner bevel
    depth = 65
    direction = 0                  #Up
    size = 17
    soften = 16
    angle = -40.0
    altitude = 30.0
    glosscontour = 4                #Cove-Shallow
    highlightcolor = ("#ffffff")
    highlightmode = 4                #LAYER_MODE_SCREEN_LEGACY
    highlightopacity = 35.0
    shadowcolor = ("#ffe7b3")
    shadowmode = 4                  #LAYER_MODE_SCREEN_LEGACY
    shadowopacity = 50.0
    surfacecontour = 6              #Half-Round
    use_texture = FALSE
    pattern = _("3D Green")
    scale = 100.0
    tex_depth = 100.0
    invert = FALSE
    merge = FALSE
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                    style, depth, direction, size, soften, angle, altitude, glosscontour, highlightcolor, highlightmode, 
                                    highlightopacity, shadowcolor, shadowmode, 
                                    shadowopacity, surfacecontour, use_texture, pattern, scale, tex_depth, invert, merge)
    bevelhighlight = pdb.gimp_image_get_layer_by_name(img, "Text-highlight")
    pdb.gimp_image_select_item(img, 2, textLayer)
    pdb.gimp_selection_invert(img)
    pdb.gimp_drawable_edit_clear(bevelhighlight)
    pdb.gimp_selection_none(img)
    
    #Rename the text
    pdb.gimp_item_set_name(textLayer, "Glazed")
    
    #Create the glaze
    if topping==1:
        glazetop = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frosted", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, glazetop, None, 0)
        pdb.gimp_drawable_fill(glazetop, FILL_TRANSPARENT)
        pdb.gimp_context_set_foreground(flacolor)
        pdb.gimp_image_select_rectangle(img, 2, 0, 151, img.width , 135)
        pdb.gimp_drawable_edit_fill(glazetop, FILL_FOREGROUND)
        pdb.gimp_selection_none(img)
        pdb.plug_in_ripple(img, glazetop, 100, 5, 1, 2, 1, TRUE, FALSE)
        pdb.gimp_layer_resize_to_image_size(glazetop)
        
        #Glaze top drop shadow
        color = "#000000"
        opacity = 20.0
        contour = 0
        noise = 0.0
        mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
        spread = 0
        size = 1
        offsetangle = 120.0
        offsetdist = 0.0
        knockout = TRUE
        merge = TRUE
        pdb.python_layerfx_drop_shadow(img, glazetop, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
        glazetop = pdb.gimp_image_get_layer_by_name(img, "Frosted")
        
        #Glaze top inner shadow
        innershadfrost = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frosted inner-shadow", 30, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, innershadfrost, None, 0)
        pdb.gimp_drawable_fill(innershadfrost, FILL_TRANSPARENT)
        pdb.gimp_image_select_item(img, 2, glazetop)
        pdb.gimp_context_set_foreground("#000000")
        pdb.gimp_drawable_fill(innershadfrost, FILL_FOREGROUND)
        pdb.gimp_selection_shrink(img, 5)
        pdb.gimp_drawable_edit_clear(innershadfrost)
        pdb.gimp_selection_none(img)
        pdb.gimp_image_select_item(img, 2, glazetop)
        pdb.plug_in_gauss(img, innershadfrost, 20, 20, 1)   #method=1
        pdb.gimp_selection_none(img)
            
        #Frostinline
        frostLine = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frostedlines", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, frostLine, None, 0)
        pdb.gimp_drawable_fill(frostLine, FILL_TRANSPARENT)
        pdb.gimp_brushes_set_brush("Chalk 03")
        pdb.gimp_context_set_brush_size(10)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        pdb.gimp_context_set_foreground(linecolor)
        pdb.gimp_context_set_dynamics("Dynamics Off")
        pdb.gimp_paintbrush_default(frostLine, 4, [0, 190, img.width, 190])
        pdb.gimp_paintbrush_default(frostLine, 4, [0, 225, img.width, 225])
        pdb.gimp_paintbrush_default(frostLine, 4, [0, 255, img.width, 255])
        pdb.plug_in_ripple(img, frostLine, 100, 5, 1, 2, 1, TRUE, FALSE)
        
        #Frostline drop shadow
        color = "#000000"
        opacity = 25.0
        contour = 0
        noise = 0.0
        mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
        spread = 0
        size = 1
        offsetangle = 120.0
        offsetdist = 0.0
        knockout = TRUE
        merge = TRUE
        pdb.python_layerfx_drop_shadow(img, frostLine, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
        frostLine = pdb.gimp_image_get_layer_by_name(img, "Frostedlines")
        
        #Frostedlines inner glow
        color = "#000000"
        opacity = 20.0
        contour = 0
        noise = 0.0
        mode = 3             #LAYER_MODE_MULTIPLY_LEGACY
        source = 1
        choke = 0.0
        size = 6
        merge = TRUE
        pdb.python_layerfx_inner_glow(img, frostLine, color, opacity, contour, noise, mode, source, choke, size, merge)
        frostLine = pdb.gimp_image_get_layer_by_name(img, "Frostedlines")
        
        #Apply the topping
        pdb.gimp_layer_resize_to_image_size(glazetop)
        pdb.gimp_image_select_item(img, 2, textLayer)
        pdb.gimp_selection_invert(img)
        pdb.gimp_selection_grow(img, 5)
        pdb.script_fu_distress_selection(img, textLayer, 
                                                    127,     #Threshold 
                                                      8,     #Spread
                                                      4,     #Granularity  
                                                      4,     #Smooth
                                                   TRUE,	  #Smooth horizontally
                                                   TRUE)	  #Smooth _vertically
        pdb.gimp_drawable_edit_clear(glazetop)
        pdb.gimp_drawable_edit_clear(innershadfrost)
        pdb.gimp_drawable_edit_clear(frostLine)
        pdb.gimp_selection_none(img)
    
    if topping==2:
        glazetop = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frosted", 100, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, glazetop, None, 0)
        pdb.gimp_drawable_fill(glazetop, FILL_TRANSPARENT)
        pdb.gimp_context_set_foreground(flacolor)
        pdb.gimp_image_select_rectangle(img, 2, 0, 151, img.width , 135)
        pdb.gimp_drawable_edit_fill(glazetop, FILL_FOREGROUND)
        pdb.gimp_selection_none(img)
        pdb.plug_in_ripple(img, glazetop, 100, 5, 1, 2, 1, TRUE, FALSE)
        pdb.gimp_layer_resize_to_image_size(glazetop)
        
        #Glaze top drop shadow
        color = "#000000"
        opacity = 20.0
        contour = 0
        noise = 0.0
        mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
        spread = 0
        size = 1
        offsetangle = 120.0
        offsetdist = 0.0
        knockout = TRUE
        merge = TRUE
        pdb.python_layerfx_drop_shadow(img, glazetop, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
        glazetop = pdb.gimp_image_get_layer_by_name(img, "Frosted")
        
        #Glaze top inner shadow
        innershadfrost = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frosted inner-shadow", 30, LAYER_MODE_NORMAL)
        pdb.gimp_image_insert_layer(img, innershadfrost, None, 0)
        pdb.gimp_drawable_fill(innershadfrost, FILL_TRANSPARENT)
        pdb.gimp_image_select_item(img, 2, glazetop)
        pdb.gimp_context_set_foreground("#000000")
        pdb.gimp_drawable_fill(innershadfrost, FILL_FOREGROUND)
        pdb.gimp_selection_shrink(img, 5)
        pdb.gimp_drawable_edit_clear(innershadfrost)
        pdb.gimp_selection_none(img)
        pdb.gimp_image_select_item(img, 2, glazetop)
        pdb.plug_in_gauss(img, innershadfrost, 20, 20, 1)   #method=1
        pdb.gimp_selection_none(img)
        
        #Sprinkles 
        pdb.gimp_image_select_item(img, 2, glazetop)
        pdb.gimp_context_set_gradient("Full saturation spectrum CCW")
        pdb.gimp_brushes_set_brush("2. Block 01")
        pdb.gimp_context_set_brush_size(10)
        pdb.gimp_context_set_brush_aspect_ratio(3.16)
        pdb.gimp_context_set_brush_angle(0)
        pdb.gimp_context_set_brush_default_hardness()
        pdb.gimp_context_set_brush_spacing(0.01)
        pdb.gimp_context_set_brush_force(1)
        pdb.gimp_context_set_gradient("Full saturation spectrum CCW")
        pdb.gimp_context_set_dynamics("Confetti")
        pdb.python_fu_density_brush_fill(img, glazetop, sprinkle_amnt)
        pdb.gimp_selection_none(img)
        sprinkles = img.active_layer
        pdb.gimp_item_set_name(sprinkles, "Sprinkles")
        
        #Sprinkles drop shadow
        color = "#000000"
        opacity = 20.0
        contour = 0
        noise = 0.0
        mode = 3           #LAYER_MODE_MULTIPLY_LEGACY
        spread = 0
        size = 1
        offsetangle = 120.0
        offsetdist = 0.0
        knockout = TRUE
        merge = TRUE
        pdb.python_layerfx_drop_shadow(img, sprinkles, color, opacity, contour, noise, mode, spread, size, offsetangle, offsetdist, knockout, merge)
        sprinkles = pdb.gimp_image_get_layer_by_name(img, "Sprinkles")
        
        #Sprinkles inner glow
        color = "#000000"
        opacity = 20.0
        contour = 0
        noise = 0.0
        mode = 3             #LAYER_MODE_MULTIPLY_LEGACY
        source = 1
        choke = 0.0
        size = 6
        merge = TRUE
        pdb.python_layerfx_inner_glow(img, sprinkles, color, opacity, contour, noise, mode, source, choke, size, merge)
        sprinkles = pdb.gimp_image_get_layer_by_name(img, "Sprinkles")
        
        #Apply the topping
        pdb.gimp_layer_resize_to_image_size(glazetop)
        pdb.gimp_image_select_item(img, 2, textLayer)
        pdb.gimp_selection_invert(img)
        pdb.gimp_selection_grow(img, 5)
        pdb.script_fu_distress_selection(img, textLayer, 
                                                    127,     #Threshold 
                                                      8,     #Spread
                                                      4,     #Granularity  
                                                      4,     #Smooth
                                                   TRUE,	  #Smooth horizontally
                                                   TRUE)	  #Smooth _vertically
        pdb.gimp_drawable_edit_clear(glazetop)
        pdb.gimp_drawable_edit_clear(innershadfrost)
        pdb.gimp_drawable_edit_clear(sprinkles)
        pdb.gimp_selection_none(img)
        pdb.gimp_context_set_dynamics("Dynamics Off")
 
    #Resets the previous user defaults
    pdb.gimp_context_pop() 
    
register(
    "pm_donut_text",
    "Creates a donuts text",
    "Creates a delicious donut text",
    "Pocholo",
    "Pocholo",
    "2022",
    "Donut Text",
    "",
    [
    (PF_STRING, "customtext", "Text string", "DONUTS"),
    (PF_FONT, "fontname", "Font name", "Franks,"),
    (PF_COLOR, "backColor", "Plaid Color", (144, 0, 0)),
    (PF_OPTION, "topping", "Topping Kind:", 0, ("Glaze", "Frosted", "Sprinkles")),
    (PF_COLOR, "flacolor", "Flavor Color", (240, 122, 168)),
    (PF_COLOR, "linecolor", "Frost Lines Color ", (255, 255, 255)),
    (PF_ADJUSTMENT, "sprinkle_amnt", "Sprinkle Amount", 10, (1, 50, 1)),
    ],
    [],
    pm_donut_text, menu="<Image>/Python-Fu/Text Effects",
    domain=("gimp20", gimp.locale_directory))
main()    